#!/usr/bin/perl

$usage = <<EOF ;
usage: polarmat.pl [-S] [-s <size>] [-a <one_sector_angle>] [-t <track_width] [-l] [-p] [-z <replace_zero>] [-c] <theta_rho_value_file>\n"
	-S : skip alternating tracks (even # tracks).
	-s <size> : set size (matrix)
	-l 	: log 
	-p 	: probability
	-z <replace_zero> : replace 0.0 with this value.
	-a <one_sector_angle> :  angle of one sector
	-t <track_width> : track width in pixels -- image scales to this
	-n <num_tracks>	: number of tracks
	-c	: cumulative sum at each sector instead of the default
		  replace by max.
EOF


require "getopts.pl";
&Getopts('Ss:lpz:a:t:c');

if ($#ARGV != 0) {
	print $usage;
	exit;
}

$track_width = ($opt_t)?$opt_t:15;
$sector_size = ($opt_a)?$opt_a:3;
$sector_per_track = int(360/$sector_size);	# number of sectors per track


$filename = $ARGV[0];

#----------------------------------------
# Get data in hash %data
#----------------------------------------
open(FP,"$filename");

$data_count = 0;
$max_rho = 0.0;
$max_val = 0.0;
$sum = 0.0;
$min_val = 9999.9;
while (<FP>) {

	if (/^#/) {
		next;
	}
	chop;
	$_ =~ s/^\ //g;
	($theta,$rho,$val) = split(/[\s]+/,$_);

	$theta = int($theta);
  	$theta = $theta - ($theta % $sector_size);

  	if ( $opt_S) { $rho *= 2; }

	if ($val<0.001 && $opt_z>0) {
		$val = $opt_z;
	}

	$data{"$theta $rho"} = $val;

	if ($rho>$max_rho) {
		$max_rho = $rho;
	}
	if ($val>$max_val) {
		$max_val = $val;
	}
	if ($val<$min_val) {
		$min_val = $val;
	}

	$data_count ++;

	$sum += $val;

}

close(FP);

$num_tracks = ($opt_n)?$opt_n:$max_rho;

#----------------------------------------
# Determine matrix size, initialize
#----------------------------------------
if ($opt_s) {
	$size = $opt_s * $track_width;
} else {
	$size = int(2.3 * $num_tracks * $track_width);
}
$cent = int($size/2);
$err = 0;
$hits=0;

for ($i=0; $i<$size; ++$i) {
	for ($j=0; $j<$size; ++$j) {
		$matrix[$i][$j] = 0.0;
	}
}

for ($i=0; $i<$size; ++$i) {

	for ($j=0; $j<$size; ++$j) {

		my $id = &in_sector($i,$j,$cent);
		my ($theta,$rho) = split(/[\s]+/,$id);
		if (exists($data{$id})) {
			if ($opt_c) {
				$matrix[$i][$j] += $data{$id};
			} else {
				if ($matrix[$i][$j]<$data{$id}) {
					$matrix[$i][$j] = $data{$id};
				}
			}
			if ($matrix[$i][$j]<=$max_val/180.0) {
				$matrix[$i][$j] = $max_val/180.0;
			}
			$hits ++;
		} else {
			if ($rho%2==0 && $rho<=$max_rho) {
				$matrix[$i][$j] = $max_val/180.0;
			}
			$err ++;
		}
	}

}

for ($j=0; $j<$size; ++$j) {
	for ($i=0; $i<$size; ++$i) {
		$val = $matrix[$i][$j];
		if ($opt_p) {
			$val = $val / $sum;
		}
		if ($opt_l) {
			if($val!=0.0) {
				$log=log($val)/log(10.0);
			} else {
				$log = 0.0;
			}
			print "$log ";
		} else {
			print "$val ";
		}
	}
	print "\n";
}

print STDERR "size = $size\n";
print STDERR "errors = $err, hits = $hits ($data_count)\n";

#----------------------------------------------------------------------------
#
# sub in_sector(x,y,center) : determine which sector (x,y) is in
#
#----------------------------------------------------------------------------
sub in_sector {

  my ($x,$y,$center) = @_;

  # $center is the center coordinate in cartesian
  # First, calculate rho 
  my $rho = sqrt(($x-$center)**2+($y-$center)**2);
  $rho = int($rho/$track_width);

  # Next, calculate theta
  my $theta = int(atan2(($y-$center),($x-$center))*180.0/3.141592);
  $theta = $theta - ($theta % $sector_size);
  if ($theta<0) { 
	$theta += 360;
  }

  return "$theta $rho";

}

